import { cloneDeep, keys, omit, pick, pickBy } from "lodash-es";
import type { Ref, UnwrapRef } from "vue";
import type { LocationQueryValue } from "vue-router";

export type UseFilters<TFilters extends Record<string, any> = Record<string, any>> = TFilters & {
  page: string | LocationQueryValue[] | null;
  per_page: string | LocationQueryValue[] | null;
  order_by: string | LocationQueryValue[] | null;
  sort: string | LocationQueryValue[] | null;
  data: () => Record<string, any>;
  changePage: (page: number | null) => void;
  clear: () => void;
  changePerPage: (perPage: number | null) => void;
};

export default function useFilters<TFilters extends Record<string, any>>(value: TFilters, skip?: (keyof TFilters)[]): Ref<UnwrapRef<UseFilters<TFilters>>> {
  const router = useRouter();
  const route = useRoute();
  const defaults = ref<TFilters>(cloneDeep(value));

  const data = (): Record<string, any> => {
    return omit<Record<string, any>>(filters.value, keys(pick(filters.value, ["clear", "changePage", "changePerPage", "data"])));
  };
  const changePage = (page: number | null) => {
    filters.value.page = page as null;
  };
  const changePerPage = (perPage: number | null) => {
    filters.value.per_page = perPage as null;
  };
  const clear = () => {
    Object.keys(defaults.value).map((item: keyof TFilters) => ((filters.value as TFilters)[item] = (defaults.value as TFilters)[item]));
  };

  const filters = ref({
    ...value,
    ...(pick(route.query ?? {}, keys(omit(value, skip ?? []))) as TFilters),
    page: route.query?.page ?? null,
    per_page: route.query?.per_page ?? null,
    order_by: route.query?.order_by ?? null,
    sort: route.query?.sort ?? null,
    data,
    clear,
    changePage,
    changePerPage,
  });

  watch(filters, () => router.replace({ query: pickBy(omit(filters.value.data(), keys(pick(filters.value, [...(skip ?? [])])))) }), { deep: true });

  return filters;
}
